package sun.rmi.transport.proxy;

import java.security.AccessControlContext;
import sun.security.action.GetBooleanAction;
import java.rmi.server.LogStream;
import java.net.ServerSocket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.net.NoRouteToHostException;
import java.io.IOException;
import java.io.InterruptedIOException;
import sun.rmi.runtime.NewThreadAction;
import java.net.Socket;
import sun.security.action.GetLongAction;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.Vector;
import java.util.Hashtable;
import sun.rmi.runtime.Log;
import java.rmi.server.RMISocketFactory;

public class RMIMasterSocketFactory extends RMISocketFactory
{
    static int logLevel;
    static final Log proxyLog;
    private static long connectTimeout;
    private static final boolean eagerHttpFallback;
    private Hashtable<String, RMISocketFactory> successTable;
    private static final int MaxRememberedHosts = 64;
    private Vector<String> hostList;
    protected RMISocketFactory initialFactory;
    protected Vector<RMISocketFactory> altFactoryList;
    
    private static String getLogLevel() {
        return (String)AccessController.doPrivileged(new GetPropertyAction("sun.rmi.transport.proxy.logLevel"));
    }
    
    private static long getConnectTimeout() {
        return (long)AccessController.doPrivileged(new GetLongAction("sun.rmi.transport.proxy.connectTimeout", 15000L));
    }
    
    public RMIMasterSocketFactory() {
        this.successTable = new Hashtable();
        this.hostList = new Vector(64);
        this.initialFactory = new RMIDirectSocketFactory();
        this.altFactoryList = new Vector(2);
        boolean n = false;
        try {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("http.proxyHost"));
            if (s == null) {
                s = (String)AccessController.doPrivileged(new GetPropertyAction("proxyHost"));
            }
            if (!((String)AccessController.doPrivileged(new GetPropertyAction("java.rmi.server.disableHttp", "true"))).equalsIgnoreCase("true") && s != null && s.length() > 0) {
                n = true;
            }
        }
        catch (Exception ex) {}
        if (!(n == 0)) {
            this.altFactoryList.addElement(new RMIHttpToPortSocketFactory());
            this.altFactoryList.addElement(new RMIHttpToCGISocketFactory());
        }
    }
    
    @Override
    public Socket createSocket(String str, int i) throws IOException {
        if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
            RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "host: " + str + ", port: " + i);
        }
        if (this.altFactoryList.size() == 0) {
            return this.initialFactory.createSocket(str, i);
        }
        RMISocketFactory obj = (RMISocketFactory)this.successTable.get(str);
        if (obj != null) {
            if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "previously successful factory found: " + obj);
            }
            return obj.createSocket(str, i);
        }
        else {
            Socket socket = null;
            Socket socket2 = null;
            RMIMasterSocketFactory.AsyncConnector runnable = new RMIMasterSocketFactory.AsyncConnector(this.initialFactory, str, i, AccessController.getContext());
            UnknownHostException | NoRouteToHostException ex = null;
            try {
                synchronized (runnable) {
                    ((Thread)AccessController.doPrivileged(new NewThreadAction(runnable, "AsyncConnector", true))).start();
                    try {
                        long n = System.currentTimeMillis();
                        long n2 = n + RMIMasterSocketFactory.connectTimeout;
                        do {
                            runnable.wait(n2 - n);
                            socket = this.checkConnector(runnable);
                            if (socket != null) {
                                break;
                            }
                            n = System.currentTimeMillis();
                        } while (n < n2);
                    }
                    catch (InterruptedException ex2) {
                        throw new InterruptedIOException("interrupted while waiting for connector");
                    }
                }
                if (socket == null) {
                    throw new NoRouteToHostException("connect timed out: " + str);
                }
                RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "direct socket connection successful");
                if (ex != null) {
                    if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "direct socket connection failed: ", ex);
                    }
                    int j = 0;
                    while (j < this.altFactoryList.size()) {
                        obj = (RMISocketFactory)this.altFactoryList.elementAt(j);
                        if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                            RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "trying with factory: " + obj);
                        }
                        Label_0562: {
                            try {
                                try (final Socket socket3 = obj.createSocket(str, i)) {
                                    socket3.getInputStream().read();
                                }
                                break Label_0562;
                            }
                            catch (IOException ex3) {
                                if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                                    RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory failed: ", ex3);
                                }
                            }
                            ++j;
                            continue;
                        }
                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory succeeded");
                        try {
                            socket2 = obj.createSocket(str, i);
                        }
                        catch (IOException ex4) {}
                        break;
                    }
                }
                return socket;
            }
            catch (UnknownHostException | NoRouteToHostException ex5) {
                ex = ex5;
                if (ex != null) {
                    if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "direct socket connection failed: ", ex);
                    }
                    int k = 0;
                    while (k < this.altFactoryList.size()) {
                        obj = (RMISocketFactory)this.altFactoryList.elementAt(k);
                        if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                            RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "trying with factory: " + obj);
                        }
                        Label_0847: {
                            try {
                                try (final Socket socket4 = obj.createSocket(str, i)) {
                                    socket4.getInputStream().read();
                                }
                                break Label_0847;
                            }
                            catch (IOException ex6) {
                                if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                                    RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory failed: ", ex6);
                                }
                            }
                            ++k;
                            continue;
                        }
                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory succeeded");
                        try {
                            socket2 = obj.createSocket(str, i);
                        }
                        catch (IOException ex7) {}
                        break;
                    }
                }
            }
            catch (SocketException ex8) {
                if (RMIMasterSocketFactory.eagerHttpFallback) {
                    ex = ex8;
                    if (ex != null) {
                        if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                            RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "direct socket connection failed: ", ex);
                        }
                        int k = 0;
                        while (k < this.altFactoryList.size()) {
                            obj = (RMISocketFactory)this.altFactoryList.elementAt(k);
                            if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                                RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "trying with factory: " + obj);
                            }
                            Label_1144: {
                                try {
                                    try (final Socket socket5 = obj.createSocket(str, i)) {
                                        socket5.getInputStream().read();
                                    }
                                    break Label_1144;
                                }
                                catch (IOException ex9) {
                                    if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory failed: ", ex9);
                                    }
                                }
                                ++k;
                                continue;
                            }
                            RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory succeeded");
                            try {
                                socket2 = obj.createSocket(str, i);
                            }
                            catch (IOException ex10) {}
                            break;
                        }
                    }
                }
                else {
                    throw ex8;
                }
            }
            finally {
                if (ex != null) {
                    if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "direct socket connection failed: ", ex);
                    }
                    int l = 0;
                    while (l < this.altFactoryList.size()) {
                        obj = (RMISocketFactory)this.altFactoryList.elementAt(l);
                        if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                            RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "trying with factory: " + obj);
                        }
                        Label_1425: {
                            try {
                                try (final Socket socket6 = obj.createSocket(str, i)) {
                                    socket6.getInputStream().read();
                                }
                                break Label_1425;
                            }
                            catch (IOException ex11) {
                                if (RMIMasterSocketFactory.proxyLog.isLoggable(Log.BRIEF)) {
                                    RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory failed: ", ex11);
                                }
                            }
                            ++l;
                            continue;
                        }
                        RMIMasterSocketFactory.proxyLog.log(Log.BRIEF, "factory succeeded");
                        try {
                            socket2 = obj.createSocket(str, i);
                        }
                        catch (IOException ex12) {}
                        break;
                    }
                }
            }
            synchronized (this.successTable) {
                try {
                    synchronized (runnable) {
                        socket = this.checkConnector(runnable);
                    }
                    if (socket != null) {
                        if (socket2 != null) {
                            socket2.close();
                        }
                        return socket;
                    }
                    else {
                        runnable.notUsed();
                    }
                }
                catch (UnknownHostException | NoRouteToHostException ex13) {
                    ex = ex13;
                }
                catch (SocketException ex14) {
                    if (RMIMasterSocketFactory.eagerHttpFallback) {
                        ex = ex14;
                    }
                    else {
                        throw ex14;
                    }
                }
                if (socket2 != null) {
                    this.rememberFactory(str, obj);
                    return socket2;
                }
                throw ex;
            }
        }
    }
    
    void rememberFactory(String s, RMISocketFactory value) {
        synchronized (this.successTable) {
            while (this.hostList.size() >= 64) {
                this.successTable.remove(this.hostList.elementAt(0));
                this.hostList.removeElementAt(0);
            }
            this.hostList.addElement(s);
            this.successTable.put(s, value);
        }
    }
    
    Socket checkConnector(RMIMasterSocketFactory.AsyncConnector asyncConnector) throws IOException {
        Exception access$000 = RMIMasterSocketFactory.AsyncConnector.access$000(asyncConnector);
        if (access$000 != null) {
            access$000.fillInStackTrace();
            if ((access$000 instanceof IOException)) {
                throw (IOException)access$000;
            }
            if ((access$000 instanceof RuntimeException)) {
                throw (RuntimeException)access$000;
            }
            throw new Error("internal error: unexpected checked exception: " + access$000.toString());
        }
        else {
            return RMIMasterSocketFactory.AsyncConnector.access$100(asyncConnector);
        }
    }
    
    @Override
    public ServerSocket createServerSocket(int n) throws IOException {
        return this.initialFactory.createServerSocket(n);
    }
    
    static {
        RMIMasterSocketFactory.logLevel = LogStream.parseLevel(getLogLevel());
        proxyLog = Log.getLog("sun.rmi.transport.tcp.proxy", "transport", RMIMasterSocketFactory.logLevel);
        RMIMasterSocketFactory.connectTimeout = getConnectTimeout();
        eagerHttpFallback = (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.rmi.transport.proxy.eagerHttpFallback"));
    }
    
    private class AsyncConnector implements Runnable
    {
        private RMISocketFactory factory;
        private String host;
        private int port;
        private AccessControlContext acc;
        private Exception exception;
        private Socket socket;
        private boolean cleanUp;
        
        AsyncConnector(RMISocketFactory factory, String s, int n, AccessControlContext acc) {
            this.exception = null;
            this.socket = null;
            this.cleanUp = false;
            this.factory = factory;
            this.host = s;
            this.port = n;
            this.acc = acc;
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkConnect(s, n);
            }
        }
        
        @Override
        public void run() {
            try {
                Socket socket = this.factory.createSocket(this.host, this.port);
                synchronized (this) {
                    this.socket = socket;
                    this.notify();
                }
                RMIMasterSocketFactory.this.rememberFactory(this.host, this.factory);
                synchronized (this) {
                    if (this.cleanUp) {
                        try {
                            this.socket.close();
                        }
                        catch (IOException ex) {}
                    }
                }
            }
            catch (Exception exception) {
                synchronized (this) {
                    this.exception = exception;
                    this.notify();
                }
            }
        }
        
        private synchronized Exception getException() {
            return this.exception;
        }
        
        private synchronized Socket getSocket() {
            return this.socket;
        }
        
        synchronized void notUsed() {
            if (this.socket != null) {
                try {
                    this.socket.close();
                }
                catch (IOException ex) {}
            }
            this.cleanUp = true;
        }
        
        static Exception access$000(RMIMasterSocketFactory.AsyncConnector asyncConnector) {
            return asyncConnector.getException();
        }
        
        static Socket access$100(RMIMasterSocketFactory.AsyncConnector asyncConnector) {
            return asyncConnector.getSocket();
        }
    }
}
